// src/index.ts
import { blue, cyan, green, magenta, red, reset, yellow } from "kolorist";
import minimist from "minimist";
import * as fs from "node:fs";
import * as path from "node:path";
import { fileURLToPath } from "node:url";
import prompts from "prompts";
var argv = minimist(process.argv.slice(2), { string: ["_"] });
var cwd = process.cwd();
var FRAMEWORKS = [
  {
    name: "empty",
    display: "Empty",
    color: yellow,
    variants: [
      {
        name: "empty",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "empty-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  },
  {
    name: "vanilla",
    display: "Vanilla",
    color: yellow,
    variants: [
      {
        name: "vanilla",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "vanilla-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  },
  {
    name: "vue",
    display: "Vue",
    color: green,
    variants: [
      {
        name: "vue",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "vue-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  },
  {
    name: "react",
    display: "React",
    color: cyan,
    variants: [
      {
        name: "react",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "react-ts",
        display: "TypeScript",
        color: blue
      },
      {
        name: "react-swc",
        display: "JavaScript + SWC",
        color: yellow
      },
      {
        name: "react-swc-ts",
        display: "TypeScript + SWC",
        color: blue
      }
    ]
  },
  {
    name: "preact",
    display: "Preact",
    color: magenta,
    variants: [
      {
        name: "preact",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "preact-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  },
  {
    name: "svelte",
    display: "Svelte",
    color: red,
    variants: [
      {
        name: "svelte",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "svelte-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  },
  {
    name: "solid",
    display: "Solid",
    color: red,
    variants: [
      {
        name: "solid",
        display: "JavaScript",
        color: yellow
      },
      {
        name: "solid-ts",
        display: "TypeScript",
        color: blue
      }
    ]
  }
];
var TEMPLATES = FRAMEWORKS.map(
  (f) => f.variants && f.variants.map((v) => v.name) || [f.name]
).reduce((a, b) => a.concat(b), []);
var renameFiles = {
  _gitignore: ".gitignore"
};
var defaultTargetDir = "vite-project";
async function init() {
  const argTargetDir = formatTargetDir(argv._[0]);
  const argTemplate = argv.template || argv.t;
  let targetDir = argTargetDir || defaultTargetDir;
  const getProjectName = () => targetDir === "." ? path.basename(path.resolve()) : targetDir;
  let result;
  try {
    result = await prompts(
      [
        {
          type: argTargetDir ? null : "text",
          name: "projectName",
          message: reset("Project name:"),
          initial: defaultTargetDir,
          onState: (state) => {
            targetDir = formatTargetDir(state.value) || defaultTargetDir;
          }
        },
        {
          type: () => !fs.existsSync(targetDir) || isEmpty(targetDir) ? null : "confirm",
          name: "overwrite",
          message: () => (targetDir === "." ? "Current directory" : `Target directory "${targetDir}"`) + ` is not empty. Remove existing files and continue?`
        },
        {
          type: (_, { overwrite: overwrite2 }) => {
            if (overwrite2 === false) {
              throw new Error(red("\u2716") + " Operation cancelled");
            }
            return null;
          },
          name: "overwriteChecker"
        },
        {
          type: () => isValidPackageName(getProjectName()) ? null : "text",
          name: "packageName",
          message: reset("Package name:"),
          initial: () => toValidPackageName(getProjectName()),
          validate: (dir) => isValidPackageName(dir) || "Invalid package.json name"
        },
        {
          type: argTemplate && TEMPLATES.includes(argTemplate) ? null : "select",
          name: "framework",
          message: typeof argTemplate === "string" && !TEMPLATES.includes(argTemplate) ? reset(
            `"${argTemplate}" isn't a valid template. Please choose from below: `
          ) : reset("Select a framework:"),
          initial: 0,
          choices: FRAMEWORKS.map((framework2) => {
            const frameworkColor = framework2.color;
            return {
              title: frameworkColor(framework2.display || framework2.name),
              value: framework2
            };
          })
        },
        {
          type: (framework2) => framework2 && framework2.variants ? "select" : null,
          name: "variant",
          message: reset("Select a variant:"),
          choices: (framework2) => framework2.variants.map((variant2) => {
            const variantColor = variant2.color;
            return {
              title: variantColor(variant2.display || variant2.name),
              value: variant2.name
            };
          })
        }
      ],
      {
        onCancel: () => {
          throw new Error(red("\u2716") + " Operation cancelled");
        }
      }
    );
  } catch (cancelled) {
    console.log(cancelled.message);
    return;
  }
  const { framework, overwrite, packageName, variant } = result;
  const root = path.join(cwd, targetDir);
  if (overwrite) {
    emptyDir(root);
  } else if (!fs.existsSync(root)) {
    fs.mkdirSync(root, { recursive: true });
  }
  let template = variant || framework?.name || argTemplate;
  let isReactSwc = false;
  if (template.includes("-swc")) {
    isReactSwc = true;
    template = template.replace("-swc", "");
  }
  const pkgInfo = pkgFromUserAgent(process.env.npm_config_user_agent);
  const pkgManager = pkgInfo ? pkgInfo.name : "npm";
  console.log(`
Scaffolding project in ${root}...`);
  const templateDir = path.resolve(
    fileURLToPath(import.meta.url),
    "../..",
    `template-${template}`
  );
  const write = (file, content) => {
    const targetPath = path.join(root, renameFiles[file] ?? file);
    if (content) {
      fs.writeFileSync(targetPath, content);
    } else {
      copy(path.join(templateDir, file), targetPath);
    }
  };
  const files = fs.readdirSync(templateDir);
  for (const file of files.filter((f) => f !== "package.json")) {
    write(file);
  }
  const pkg = JSON.parse(
    fs.readFileSync(path.join(templateDir, `package.json`), "utf-8")
  );
  pkg.name = packageName || getProjectName();
  write("package.json", JSON.stringify(pkg, null, 2));
  if (isReactSwc) {
    setupReactSwc(root, template.endsWith("-ts"));
  }
  console.log(`
Done. Now run:
`);
  if (root !== cwd) {
    console.log(`  cd ${path.relative(cwd, root)}`);
  }
  switch (pkgManager) {
    case "yarn":
      console.log("  yarn");
      console.log("  yarn dev");
      break;
    default:
      console.log(`  ${pkgManager} install`);
      console.log(`  ${pkgManager} run dev`);
      break;
  }
  console.log();
}
function formatTargetDir(targetDir) {
  return targetDir?.trim().replace(/\/+$/g, "");
}
function copy(src, dest) {
  const stat = fs.statSync(src);
  if (stat.isDirectory()) {
    copyDir(src, dest);
  } else {
    fs.copyFileSync(src, dest);
  }
}
function isValidPackageName(projectName) {
  return /^(?:@[a-z\d\-*~][a-z\d\-*._~]*\/)?[a-z\d\-~][a-z\d\-._~]*$/.test(
    projectName
  );
}
function toValidPackageName(projectName) {
  return projectName.trim().toLowerCase().replace(/\s+/g, "-").replace(/^[._]/, "").replace(/[^a-z\d\-~]+/g, "-");
}
function copyDir(srcDir, destDir) {
  fs.mkdirSync(destDir, { recursive: true });
  for (const file of fs.readdirSync(srcDir)) {
    const srcFile = path.resolve(srcDir, file);
    const destFile = path.resolve(destDir, file);
    copy(srcFile, destFile);
  }
}
function isEmpty(path2) {
  const files = fs.readdirSync(path2);
  return files.length === 0 || files.length === 1 && files[0] === ".git";
}
function emptyDir(dir) {
  if (!fs.existsSync(dir)) {
    return;
  }
  for (const file of fs.readdirSync(dir)) {
    if (file === ".git") {
      continue;
    }
    fs.rmSync(path.resolve(dir, file), { recursive: true, force: true });
  }
}
function pkgFromUserAgent(userAgent) {
  if (!userAgent) return void 0;
  const pkgSpec = userAgent.split(" ")[0];
  const pkgSpecArr = pkgSpec.split("/");
  return {
    name: pkgSpecArr[0],
    version: pkgSpecArr[1]
  };
}
function setupReactSwc(root, isTs) {
  editFile(path.resolve(root, "package.json"), (content) => {
    return content.replace(
      /"@vitejs\/plugin-react": ".+?"/,
      `"@vitejs/plugin-react-swc": "^3.0.0"`
    );
  });
  editFile(
    path.resolve(root, `vite.config.${isTs ? "ts" : "js"}`),
    (content) => {
      return content.replace(
        "@vitejs/plugin-react",
        "@vitejs/plugin-react-swc"
      );
    }
  );
}
function editFile(file, callback) {
  const content = fs.readFileSync(file, "utf-8");
  fs.writeFileSync(file, callback(content), "utf-8");
}
init().catch((e) => {
  console.error(e);
});
//# sourceMappingURL=index.js.map